"""
Conditioner
===========
Create simple 'if this then that' style rules in your Django application. Comes with a bunch of ready to use actions
and conditions, but is also easily extensible and allows model specific actions/conditions.
"""
default_app_config = 'conditioner.apps.ConditionerAppConfig'


__title__ = 'django-conditioner'
__version__ = '0.1.0'
__author__ = 'Omni Digital'
__license__ = 'MIT'
__url__ = 'https://github.com/omni-digital/django-conditioner'
